@@ -1,3 +1,10 @@
+<?php
+
+if (!class_exists('rcube_install') || !is_object($RCI)) {
+    die("Not allowed! Please open installer/index.php instead.");
+}
+
+?>
 <form action="index.php" method="post">
 <input type="hidden" name="_step" value="2" />
 <?php
@@ -8,7 +15,6 @@ $RCI->load_defaults();
 // register these boolean fields
 $RCI->bool_config_props = array(
   'ip_check' => 1,
-  'enable_caching' => 1,
   'enable_spellcheck' => 1,
   'auto_create_user' => 1,
   'smtp_log' => 1,
@@ -23,8 +29,8 @@ $_SESSION['allowinstaller'] = true;
 if (!empty($_POST['submit'])) {
   
   echo '<p class="notice">Copy or download the following configurations and save them in two files';
-  echo ' (names above the text box) within the <tt>'.RCMAIL_CONFIG_DIR.'</tt> directory of your Roundcube installation.<br/>';
-  echo ' Make sure that there are no characters outside the <tt>&lt;?php ?&gt;</tt> brackets when saving the files.</p>';
+  echo ' (names above the text box) within the <tt>'.RCMAIL_CONFIG_DIR.'</tt> directory of your Roundcube installation.';
+  echo '</p>';
   
   $textbox = new html_textarea(array('rows' => 16, 'cols' => 60, 'class' => "configfile"));
   
@@ -59,6 +65,30 @@ echo $input_prodname->show($RCI->getprop('product_name'));
 <div>The name of your service (used to compose page titles)</div>
 </dd>
 
+<dt class="propname">support_url</dt>
+<dd>
+<?php
+
+$input_support = new html_inputfield(array('name' => '_support_url', 'size' => 50, 'id' => "cfgsupporturl"));
+echo $input_support->show($RCI->getprop('support_url'));
+
+?>
+<div>Provide an URL where a user can get support for this Roundcube installation.<br/>PLEASE DO NOT LINK TO THE ROUNDCUBE.NET WEBSITE HERE!</div>
+<p class="hint">Enter an absolute URL (inculding http://) to a support page/form or a mailto: link.</p>
+</dd>
+
+<dt class="propname">skin_logo</dt>
+<dd>
+<?php
+
+$input_skin = new html_inputfield(array('name' => '_skin_logo', 'size' => 50, 'id' => "cfgskinlogo"));
+echo $input_skin->show($RCI->getprop('skin_logo'));
+
+?>
+<div>Custom image to display instead of the Roundcube logo.</div>
+<p class="hint">Enter a URL relative to the document root of this Roundcube installation.</p>
+</dd>
+
 <dt class="propname">temp_dir</dt>
 <dd>
 <?php
@@ -70,20 +100,6 @@ echo $input_tempdir->show($RCI->getprop('temp_dir'));
 <div>Use this folder to store temp files (must be writeable for webserver)</div>
 </dd>
 
-
-<dt class="propname">ip_check</dt>
-<dd>
-<?php
-
-$check_ipcheck = new html_checkbox(array('name' => '_ip_check', 'id' => "cfgipcheck"));
-echo $check_ipcheck->show(intval($RCI->getprop('ip_check')), array('value' => 1));
-
-?>
-<label for="cfgipcheck">Check client IP in session authorization</label><br />
-
-<p class="hint">This increases security but can cause sudden logouts when someone uses a proxy with changing IPs.</p>
-</dd>
-
 <dt class="propname">des_key</dt>
 <dd>
 <?php
@@ -97,17 +113,20 @@ echo $input_deskey->show($RCI->getprop('des_key'));
 If you enter it manually please provide a string of exactly 24 chars.</p>
 </dd>
 
-<dt class="propname">enable_caching</dt>
+<dt class="propname">ip_check</dt>
 <dd>
 <?php
 
-$check_caching = new html_checkbox(array('name' => '_enable_caching', 'id' => "cfgcache"));
-echo $check_caching->show(intval($RCI->getprop('enable_caching')), array('value' => 1));
+$check_ipcheck = new html_checkbox(array('name' => '_ip_check', 'id' => "cfgipcheck"));
+echo $check_ipcheck->show(intval($RCI->getprop('ip_check')), array('value' => 1));
 
 ?>
-<label for="cfgcache">Cache messages in local database</label><br />
+<label for="cfgipcheck">Check client IP in session authorization</label><br />
+
+<p class="hint">This increases security but can cause sudden logouts when someone uses a proxy with changing IPs.</p>
 </dd>
 
+
 <dt class="propname">enable_spellcheck</dt>
 <dd>
 <?php
@@ -244,15 +263,10 @@ echo $input_syslogfacility->show($RCI->getprop('syslog_facility'), LOG_USER);
 <p>Database settings for read/write operations:</p>
 <?php
 
-require_once 'MDB2.php';
-
-$supported_dbs = array('MySQL' => 'mysql', 'MySQLi' => 'mysqli',
-    'PgSQL' => 'pgsql', 'SQLite' => 'sqlite');
-
 $select_dbtype = new html_select(array('name' => '_dbtype', 'id' => "cfgdbtype"));
-foreach ($supported_dbs AS $database => $ext) {
+foreach ($RCI->supported_dbs as $database => $ext) {
     if (extension_loaded($ext)) {
-        $select_dbtype->add($database, $ext);
+        $select_dbtype->add($database, substr($ext, 4));
     }
 }
 
@@ -261,7 +275,7 @@ $input_dbname = new html_inputfield(array('name' => '_dbname', 'size' => 20, 'id
 $input_dbuser = new html_inputfield(array('name' => '_dbuser', 'size' => 20, 'id' => "cfgdbuser"));
 $input_dbpass = new html_passwordfield(array('name' => '_dbpass', 'size' => 20, 'id' => "cfgdbpass"));
 
-$dsnw = MDB2::parseDSN($RCI->getprop('db_dsnw'));
+$dsnw = rcube_db::parse_dsn($RCI->getprop('db_dsnw'));
 
 echo $select_dbtype->show($RCI->is_post ? $_POST['_dbtype'] : $dsnw['phptype']);
 echo '<label for="cfgdbtype">Database type</label><br />';
@@ -512,27 +526,34 @@ echo $input_skin->show($RCI->getprop('skin'));
 <div>Name of interface skin (folder in /skins)</div>
 </dd>
 
-<dt class="propname">skin_logo</dt>
+<dt class="propname">mail_pagesize <span class="userconf">*</span></dt>
 <dd>
 <?php
 
-$input_skin = new html_inputfield(array('name' => '_skin_logo', 'size' => 50, 'id' => "cfgskinlogo"));
-echo $input_skin->show($RCI->getprop('skin_logo'));
+$pagesize = $RCI->getprop('mail_pagesize');
+if (!$pagesize) {
+    $pagesize = $RCI->getprop('pagesize');
+}
+$input_pagesize = new html_inputfield(array('name' => '_mail_pagesize', 'size' => 6, 'id' => "cfgmailpagesize"));
+echo $input_pagesize->show($pagesize);
 
 ?>
-<div>Custom image to display instead of the Roundcube logo.</div>
-<p class="hint">Enter a URL relative to the document root of this Roundcube installation.</p>
+<div>Show up to X items in the mail messages list view.</div>
 </dd>
 
-<dt class="propname">pagesize <span class="userconf">*</span></dt>
+<dt class="propname">addressbook_pagesize <span class="userconf">*</span></dt>
 <dd>
 <?php
 
-$input_pagesize = new html_inputfield(array('name' => '_pagesize', 'size' => 6, 'id' => "cfgpagesize"));
-echo $input_pagesize->show($RCI->getprop('pagesize'));
+$pagesize = $RCI->getprop('addressbook_pagesize');
+if (!$pagesize) {
+    $pagesize = $RCI->getprop('pagesize');
+}
+$input_pagesize = new html_inputfield(array('name' => '_addressbook_pagesize', 'size' => 6, 'id' => "cfgabookpagesize"));
+echo $input_pagesize->show($pagesize);
 
 ?>
-<div>Show up to X items in list view.</div>
+<div>Show up to X items in the contacts list view.</div>
 </dd>
 
 <dt class="propname">prefer_html <span class="userconf">*</span></dt>
