@@ -7,6 +7,7 @@
  * to move messages to a (user selectable) archive folder.
  *
  * @version @package_version@
+ * @license GNU GPLv3+
  * @author Andre Rodier, Thomas Bruederli
  */
 class archive extends rcube_plugin
@@ -20,16 +21,20 @@ class archive extends rcube_plugin
     // There is no "Archived flags"
     // $GLOBALS['IMAP_FLAGS']['ARCHIVED'] = 'Archive';
     if ($rcmail->task == 'mail' && ($rcmail->action == '' || $rcmail->action == 'show')
-      && ($archive_folder = $rcmail->config->get('archive_mbox'))) {
+        && ($archive_folder = $rcmail->config->get('archive_mbox'))) {
       $skin_path = $this->local_skin_path();
-      
+      if (is_file($this->home . "/$skin_path/archive.css"))
+        $this->include_stylesheet("$skin_path/archive.css");
+
       $this->include_script('archive.js');
       $this->add_texts('localization', true);
       $this->add_button(
         array(
+            'type' => 'link',
+            'label' => 'buttontext',
             'command' => 'plugin.archive',
-            'imagepas' => $skin_path.'/archive_pas.png',
-            'imageact' => $skin_path.'/archive_act.png',
+            'class' => 'button buttonPas archive disabled',
+            'classact' => 'button archive',
             'width' => 32,
             'height' => 32,
             'title' => 'buttontitle',
@@ -42,12 +47,11 @@ class archive extends rcube_plugin
 
       // set env variable for client
       $rcmail->output->set_env('archive_folder', $archive_folder);
-      $rcmail->output->set_env('archive_folder_icon', $this->url($skin_path.'/foldericon.png'));
 
       // add archive folder to the list of default mailboxes
-      if (($default_folders = $rcmail->config->get('default_imap_folders')) && !in_array($archive_folder, $default_folders)) {
+      if (($default_folders = $rcmail->config->get('default_folders')) && !in_array($archive_folder, $default_folders)) {
         $default_folders[] = $archive_folder;
-        $rcmail->config->set('default_imap_folders', $default_folders);
+        $rcmail->config->set('default_folders', $default_folders);
       }  
     }
     else if ($rcmail->task == 'settings') {
@@ -63,9 +67,10 @@ class archive extends rcube_plugin
   {
     $rcmail = rcmail::get_instance();
     $archive_folder = $rcmail->config->get('archive_mbox');
+    $localize_name = $rcmail->config->get('archive_localize_name', true);
 
     // set localized name for the configured archive folder
-    if ($archive_folder) {
+    if ($archive_folder && $localize_name) {
       if (isset($p['list'][$archive_folder]))
         $p['list'][$archive_folder]['name'] = $this->gettext('archivefolder');
       else // search in subfolders
