@@ -3,9 +3,13 @@
 /**
  * Sample plugin to try out some hooks.
  * This performs an automatic login if accessed from localhost
+ *
+ * @license GNU GPLv3+
+ * @author Thomas Bruederli
  */
 class autologon extends rcube_plugin
 {
+  public $task = 'login';
 
   function init()
   {
@@ -18,7 +22,7 @@ class autologon extends rcube_plugin
     $rcmail = rcmail::get_instance();
 
     // change action to login
-    if ($args['task'] == 'mail' && empty($args['action']) && empty($_SESSION['user_id']) && !empty($_GET['_autologin']) && $this->is_localhost())
+    if (empty($_SESSION['user_id']) && !empty($_GET['_autologin']) && $this->is_localhost())
       $args['action'] = 'login';
 
     return $args;
@@ -30,6 +34,8 @@ class autologon extends rcube_plugin
       $args['user'] = 'me';
       $args['pass'] = '******';
       $args['host'] = 'localhost';
+      $args['cookiecheck'] = false;
+      $args['valid'] = true;
     }
   
     return $args;
